package de.cliffbreak.varo;

import java.io.File;
import java.io.IOException;
import java.time.Instant;
import java.util.ArrayList;

import org.bukkit.configuration.file.FileConfiguration;
import org.bukkit.configuration.file.YamlConfiguration;
import org.bukkit.plugin.java.JavaPlugin;

import de.cliffbreak.varo.commands.CoordsCommand;
import de.cliffbreak.varo.commands.InfoCommand;
import de.cliffbreak.varo.commands.VaroCommand;
import de.cliffbreak.varo.commands.VaroTabCompleter;
import de.cliffbreak.varo.database.Database;
import de.cliffbreak.varo.discord.DiscordBot;
import de.cliffbreak.varo.listeners.BannedActionListener;
import de.cliffbreak.varo.listeners.BannedItemListener;
import de.cliffbreak.varo.listeners.ChatListener;
import de.cliffbreak.varo.listeners.EntityRegainHealthListener;
import de.cliffbreak.varo.listeners.InventoryClickListener;
import de.cliffbreak.varo.listeners.PlayerClientOptionsChangeListener;
import de.cliffbreak.varo.listeners.PlayerDeathListener;
import de.cliffbreak.varo.listeners.PlayerInteractNPCListener;
import de.cliffbreak.varo.listeners.PlayerJoinQuitListener;
import de.cliffbreak.varo.listeners.PlayerLoginListener;
import de.cliffbreak.varo.listeners.PlayerResourcePackStatusListener;
import de.cliffbreak.varo.managers.BorderManager;
import de.cliffbreak.varo.managers.GameManager;
import de.cliffbreak.varo.managers.GameTimeManager;
import de.cliffbreak.varo.managers.NPCManager;
import de.cliffbreak.varo.managers.TeamManager;
import de.cliffbreak.varo.utils.BanUtils;
import de.cliffbreak.varo.utils.LangUtils;
import de.cliffbreak.varo.utils.MessageUtils;
import de.cliffbreak.varo.utils.PlayerCache;
import de.cliffbreak.varo.utils.PlayerUtils;
import de.cliffbreak.varo.utils.VaroUtils;

public class Varo extends JavaPlugin {

    private File configurationFile;
    private FileConfiguration config;
    private Database database;
    private TeamManager teamManager;
    private PlayerCache playerCache;
    private NPCManager npcManager;
    private BanUtils banUtils;
    private VaroUtils varoUtils;
    private MessageUtils messageUtils;
    private GameManager gameManager;
    private GameTimeManager gameTimeManager;
    private PlayerUtils playerUtils;
    private BorderManager borderManager;
    private LangUtils langUtils;
    private DiscordBot discordBot;

    private boolean canCreatePortal = false;

    @Override
    public void onEnable() {

        this.getLogger().info("CliffbreakVaro is starting...");

        this.configurationFile = new File("plugins/CliffbreakVaro", "config.yml");
        this.config = YamlConfiguration.loadConfiguration(this.configurationFile);

        this.config.addDefault("Varo.Start", 1590393600000f);
        this.config.addDefault("Varo.Debug", false);
        this.config.addDefault("Varo.Discord.Token", "insert.token.here");
        this.config.addDefault("Varo.Discord.Channel", "numeric.channel.id");
        this.config.addDefault("Varo.Discord.BroadcastChannel", "numeric.channel.id");
        this.config.addDefault("Varo.RetroHealthRegen", false);
        this.config.addDefault("Varo.Bans", new ArrayList<String>());
        this.config.addDefault("Varo.Border.X", 0);
        this.config.addDefault("Varo.Border.Z", 0);
        this.config.addDefault("Varo.Border.Size", 1250 * 2); // 1.250x1.250
        this.config.addDefault("Varo.Border.NextSize", this.config.getDefaults().get("Varo.Border.Size"));
        this.config.addDefault("Varo.Border.LastUpdate", Instant.now().toEpochMilli());
        this.config.options().copyDefaults(true);
        this.saveConfig();

        this.database = new Database();
        this.teamManager = new TeamManager();
        this.playerCache = new PlayerCache(this);
        this.npcManager = new NPCManager(this);
        this.banUtils = new BanUtils(this);
        this.varoUtils = new VaroUtils(this);
        this.messageUtils = new MessageUtils(this);
        this.gameManager = new GameManager(this);
        this.gameTimeManager = new GameTimeManager(this);
        this.playerUtils = new PlayerUtils(this);
        this.borderManager = new BorderManager(this);
        this.langUtils = new LangUtils();

        this.getServer().getPluginManager().registerEvents(new PlayerLoginListener(this), this);
        this.getServer().getPluginManager().registerEvents(new PlayerJoinQuitListener(this), this);
        this.getServer().getPluginManager().registerEvents(new PlayerClientOptionsChangeListener(this), this);
        this.getServer().getPluginManager().registerEvents(new PlayerResourcePackStatusListener(this), this);
        this.getServer().getPluginManager().registerEvents(new ChatListener(this), this);
        this.getServer().getPluginManager().registerEvents(new EntityRegainHealthListener(this), this);
        this.getServer().getPluginManager().registerEvents(new PlayerDeathListener(this), this);
        this.getServer().getPluginManager().registerEvents(new BannedItemListener(this), this);
        this.getServer().getPluginManager().registerEvents(new BannedActionListener(this), this);
        this.getServer().getPluginManager().registerEvents(new PlayerInteractNPCListener(this), this);
        this.getServer().getPluginManager().registerEvents(new InventoryClickListener(this), this);

        this.getCommand("varo").setExecutor(new VaroCommand(this));
        this.getCommand("varo").setTabCompleter(new VaroTabCompleter(this));
        this.getCommand("coords").setExecutor(new CoordsCommand(this));
        this.getCommand("info").setExecutor(new InfoCommand(this));

        this.varoUtils.init();

        this.discordBot = new DiscordBot(this);
    }

    @Override
    public void onDisable() {
        this.getLogger().info("Stopping CliffbreakVaro!");
        this.database.shutdown();
        try {
            this.discordBot.shutdown();
        } catch (final NoClassDefFoundError err) {

        }
    }

    @Override
    public FileConfiguration getConfig() {
        return this.config;
    }

    @Override
    public void saveConfig() {
        try {
            this.getLogger().info("Saving Config...");
            this.config.save(this.configurationFile);
        } catch (final IOException e) {
            e.printStackTrace();
        }
    }

    public Database getDatabase() {
        return this.database;
    }

    public TeamManager getTeamManager() {
        return this.teamManager;
    }

    public PlayerCache getPlayerCache() {
        return this.playerCache;
    }

    public NPCManager getNPCManager() {
        return this.npcManager;
    }

    public BanUtils getBanUtils() {
        return this.banUtils;
    }

    public MessageUtils getMessageUtils() {
        return this.messageUtils;
    }

    public GameManager getGameManager() {
        return this.gameManager;
    }

    public GameTimeManager getGameTimeManager() {
        return this.gameTimeManager;
    }

    public PlayerUtils getPlayerUtils() {
        return this.playerUtils;
    }

    public BorderManager getBorderManager() {
        return this.borderManager;
    }

    public LangUtils getLangUtils() {
        return this.langUtils;
    }

    public DiscordBot getDiscordBot() {
        return this.discordBot;
    }

    public void setCanCreatePortal(final boolean canCreatePortal) {
        this.canCreatePortal = canCreatePortal;
    }

    public boolean canCreatePortal() {
        return this.canCreatePortal;
    }
}
