package de.cliffbreak.varo.commands;

import java.io.IOException;
import java.util.HashMap;
import java.util.UUID;

import org.bukkit.Bukkit;
import org.bukkit.command.Command;
import org.bukkit.command.CommandExecutor;
import org.bukkit.command.CommandSender;
import org.bukkit.entity.Player;
import org.json.simple.JSONArray;
import org.json.simple.JSONObject;
import org.json.simple.JSONValue;
import org.json.simple.parser.ParseException;

import de.cliffbreak.varo.Varo;
import de.cliffbreak.varo.utils.WebUtils;

public class VaroCommand implements CommandExecutor {

    private final Varo plugin;
    private final HashMap<UUID, TeamPlayerId> addSpawnsCounter = new HashMap<UUID, TeamPlayerId>();

    public VaroCommand(final Varo plugin) {
        this.plugin = plugin;
    }

    @Override
    public boolean onCommand(final CommandSender sender, final Command command, final String label,
            final String[] args) {
        if (!sender.hasPermission("varo.admin")) {
            sender.sendMessage("§c§lFehler: §r§cNicht genügend Rechte für dieses Kommando.");
            return true;
        }
        if (args.length == 2 && args[0].equals("unban")) {
            try {
                final String result = WebUtils.GET("https://api.mojang.com/users/profiles/minecraft/" + args[1]);
                if (result.isEmpty()) {
                    sender.sendMessage("§c§lFehler: §r§cDer Spieler wurde nicht gefunden.");
                    return true;
                }
                final JSONObject uuidObject = (JSONObject) JSONValue.parseWithException(result);
                if (!this.plugin.getBanUtils().isBanned(uuidObject.get("id").toString())) {
                    sender.sendMessage("§c§lFehler: §r§cDer Spieler ist nicht gebannt.");
                    return true;
                }
                this.plugin.getBanUtils().removeBan(uuidObject.get("id").toString());
                sender.sendMessage("§a§lErfolg: §r§aDer Spieler ist nun nicht mehr gebannt.");
            } catch (IOException | ParseException e) {
                sender.sendMessage(
                        "§c§lFehler:§r§c beim Verbinden mit der Mojang-API ist ein Fehler aufgetreten. Eventuell sind die Mojang Server down.");
                e.printStackTrace();
            }
        } else if (args.length == 2 && args[0].equals("ban")) {
            try {
                final String result = WebUtils.GET("https://api.mojang.com/users/profiles/minecraft/" + args[1]);
                if (result.isEmpty()) {
                    sender.sendMessage("§c§lFehler: §r§cDer Spieler wurde nicht gefunden.");
                    return true;
                }
                final JSONObject uuidObject = (JSONObject) JSONValue.parseWithException(result);
                if (this.plugin.getPlayerCache().getName(uuidObject.get("id").toString()) == null) {
                    sender.sendMessage("§c§lFehler: §r§cDer Spieler war noch nie auf diesem Server.");
                    return true;
                }
                this.plugin.getBanUtils().addBan(uuidObject.get("id").toString());
                sender.sendMessage("§a§lErfolg: §r§aDer Spieler ist nun gebannt.");
            } catch (IOException | ParseException e) {
                sender.sendMessage(
                        "§c§lFehler:§r§c beim Verbinden mit der Mojang-API ist ein Fehler aufgetreten. Eventuell sind die Mojang Server down.");
                e.printStackTrace();
            }
        } else if (args.length == 2 && args[0].equals("portal")) {
            if (args[1].equals("enable")) {
                this.plugin.setCanCreatePortal(true);
                sender.sendMessage("§c§lWarnung: §r§cPortale können nun von JEDEM Spieler erstellt werden.");
            } else if (args[1].equals("disable")) {
                this.plugin.setCanCreatePortal(false);
                sender.sendMessage("§a§lErfolg: §r§aEs können nun keine Portale mehr erstellt werden..");
            } else {
                sender.sendMessage("§c§lFehler: §r§cBitte überprüfe deine Eingabe.");
            }
        } else if (args.length == 2 && args[0].equals("heal")) {
            Player p = Bukkit.getPlayerExact(args[1]);
            if (p == null) {
                sender.sendMessage("§c§lFehler: §r§cDer Spieler ist nicht online.");
                return true;
            }
            p.setHealth(20.0d);
            p.setFoodLevel(20);
            p.setSaturation(5.0f);
        } else if (args.length == 1 && args[0].equals("addspawns")) {
            if (!(sender instanceof Player)) {
                sender.sendMessage("§c§lFehler: §r§cDieses Kommando kann nur von einem Spieler ausgeführt werden.");
                return true;
            }
            final Player p = (Player) sender;
            TeamPlayerId teamPlayerId;
            boolean firstRun = false;
            if (this.addSpawnsCounter.containsKey(p.getUniqueId())) {
                teamPlayerId = this.addSpawnsCounter.get(p.getUniqueId());
            } else {
                p.sendMessage("§aDer Spawn-Erstellungs-Wizard wird gestartet...");
                teamPlayerId = new TeamPlayerId(0, 0);
                this.addSpawnsCounter.put(p.getUniqueId(), teamPlayerId);
                firstRun = true;
            }
            final JSONArray teams = this.plugin.getTeamManager().getJSONTeams();
            final JSONObject team = (JSONObject) teams.get(teamPlayerId.team);
            final JSONArray players = (JSONArray) team.get("players");

            if (!firstRun) {
                p.sendMessage("§a§lErfolg.");
                this.plugin.getTeamManager().setPlayerSpawnByKeys(teamPlayerId.team, teamPlayerId.player,
                        p.getLocation());
                if (teamPlayerId.player + 1 == players.size()) {
                    if (teamPlayerId.team + 1 == teams.size()) {
                        p.sendMessage("§a§lErfolg: §r§aAlle Spawns wurden erfolgreich gesetzt.");
                        this.addSpawnsCounter.remove(p.getUniqueId());
                        return true;
                    } else {
                        this.addSpawnsCounter.replace(p.getUniqueId(), new TeamPlayerId(teamPlayerId.team + 1, 0));
                    }
                } else {
                    this.addSpawnsCounter.replace(p.getUniqueId(),
                            new TeamPlayerId(teamPlayerId.team, teamPlayerId.player + 1));
                }
            }

            String playerName = "";
            String teamName = "";
            if (teamPlayerId.player + 1 == players.size()) {
                final JSONObject tmpTeam = (JSONObject) teams.get(teamPlayerId.team + 1);
                teamName = (String) tmpTeam.get("name");
                playerName = (String) ((JSONObject) ((JSONArray) tmpTeam.get("players")).get(0)).get("name");
            } else {
                teamName = (String) team.get("name");
                playerName = (String) ((JSONObject) players.get(teamPlayerId.player + 1)).get("name");
            }
            if (firstRun) {
                teamName = (String) team.get("name");
                playerName = (String) ((JSONObject) players.get(0)).get("name");
            }
            p.sendMessage("Begebe dich in Position um den Spawn für Spieler: §6" + playerName + "§r vom Team: §6"
                    + teamName + "§r zu setzen.\nFühre danach dieses Kommando erneut aus, um den Spawn zu setzten.");
        } else {
            sender.sendMessage("\n§6========== §b§lCliffbreak.de VARO §r§6============\n ");
            sender.sendMessage("§a/varo ban [Spieler]: §rSpieler bannen");
            sender.sendMessage("§a/varo unban [Spieler]: §rSpieler wieder auf den Server lassen");
            sender.sendMessage("§a/varo heal [Spieler]: §rSpieler heilen und Hunger auffüllen");
            sender.sendMessage("§a/varo portal [enable/disable]: §rAktiviere/Deaktiviere die Portal-Erstellung");
            sender.sendMessage("§a/varo addspawns: §rStarte den Spawn-Erstellungs-Wizard");
            sender.sendMessage("\n§6==========================================");
        }
        return true;
    }

    private class TeamPlayerId {
        public final int team;
        public final int player;

        public TeamPlayerId(final int team, final int player) {
            this.team = team;
            this.player = player;
        }
    }
}