package de.cliffbreak.varo.discord;

import java.util.EnumSet;

import javax.security.auth.login.LoginException;

import org.apache.logging.log4j.LogManager;
import org.apache.logging.log4j.core.Filter;
import org.apache.logging.log4j.core.Logger;

import de.cliffbreak.varo.Varo;
import de.cliffbreak.varo.discord.filters.JDAFilter;
import de.cliffbreak.varo.discord.listeners.MessageListener;
import net.dv8tion.jda.api.JDA;
import net.dv8tion.jda.api.JDABuilder;
import net.dv8tion.jda.api.utils.cache.CacheFlag;

public class DiscordBot {

    private final Varo plugin;
    private final JDA jda;

    public DiscordBot(final Varo plugin) {
        this.plugin = plugin;
        ((Logger) LogManager.getRootLogger()).addFilter((Filter) new JDAFilter());
        this.jda = getJDA();
    }

    public void shutdown() {
        this.jda.shutdown();
    }

    public void broadcastMessage(final String message) {
        this.jda.getTextChannelById(this.plugin.getConfig().getString("Varo.Discord.BroadcastChannel"))
                .sendMessage(message).queue();
    }

    private JDA getJDA() {
        try {
            return new JDABuilder().setAutoReconnect(true).setBulkDeleteSplittingEnabled(false)
                    .setDisabledCacheFlags(EnumSet.of(CacheFlag.ACTIVITY, CacheFlag.VOICE_STATE))
                    .setToken(this.plugin.getConfig().getString("Varo.Discord.Token")).setContextEnabled(false)
                    .addEventListeners(new MessageListener(this.plugin)).build().awaitReady();
        } catch (LoginException | InterruptedException e) {
            this.plugin.getLogger().warning("The Discord-Service didn't connect properly.");
            return null;
        }
    }

}