package de.cliffbreak.varo.discord.listeners;

import java.util.ArrayList;
import java.util.Arrays;

import de.cliffbreak.varo.Varo;
import net.dv8tion.jda.api.entities.EmbedType;
import net.dv8tion.jda.api.entities.Message;
import net.dv8tion.jda.api.entities.MessageChannel;
import net.dv8tion.jda.api.entities.MessageEmbed;
import net.dv8tion.jda.api.entities.MessageEmbed.Field;
import net.dv8tion.jda.api.events.message.MessageReceivedEvent;
import net.dv8tion.jda.api.hooks.ListenerAdapter;

public class MessageListener extends ListenerAdapter {

    final Varo plugin;

    public MessageListener(final Varo plugin) {
        this.plugin = plugin;
    }

    @Override
    public void onMessageReceived(final MessageReceivedEvent e) {
        // Don't respond to Bot accounts (and myself)
        if (e.getAuthor().isBot()) {
            return;
        }
        // Only answer in configured Channels
        final MessageChannel channel = e.getChannel();
        if (!channel.getId().equals(this.plugin.getConfig().getString("Varo.Discord.Channel"))) {
            return;
        }
        final Message message = e.getMessage();
        switch (message.getContentRaw()) {
        case "!ping":
            channel.sendMessage("Pong!").queue();
            break;
        case "!test":
            channel.sendMessage(new MessageEmbed(null, "title", "description", EmbedType.RICH, null, 0, null, null,
                    null, null, null, null, new ArrayList<Field>(Arrays.asList()))).queue();
            break;
        }
        if (message.getContentRaw().startsWith("!stats")) {
            if (message.getMentionedMembers().size() != 1)
                return;

            final String id = message.getMentionedMembers().get(0).getId();
            final String uuid = this.plugin.getTeamManager().getPlayerByDiscordId(id);
            final int kills = this.plugin.getDatabase().getKills(uuid);
            channel.sendMessage(this.plugin.getLangUtils().get("Discord.PlayerStats").replace("%id%", id)
                    .replace("%kills%", "" + kills)).queue();
        }
    }
}