package de.cliffbreak.varo.listeners;

import org.bukkit.Bukkit;
import org.bukkit.GameMode;
import org.bukkit.entity.Player;
import org.bukkit.event.EventHandler;
import org.bukkit.event.Listener;
import org.bukkit.event.entity.EntityDamageEvent;
import org.bukkit.event.entity.PlayerDeathEvent;
import org.bukkit.event.entity.EntityDamageEvent.DamageCause;
import org.bukkit.scheduler.BukkitRunnable;

import de.cliffbreak.varo.Varo;

public class PlayerDeathListener implements Listener {

    private final Varo plugin;

    public PlayerDeathListener(final Varo plugin) {
        this.plugin = plugin;
    }

    @EventHandler
    public void onPlayerDeath(final PlayerDeathEvent e) {
        new BukkitRunnable() {
            @Override
            public void run() {
                e.getEntity().kickPlayer(plugin.getLangUtils().get("Service.BannedDead"));
            }
        }.runTaskLater(this.plugin, 20);
        final EntityDamageEvent damageCause = e.getEntity().getLastDamageCause();
        final Player killer = e.getEntity().getKiller();
        if (killer != null) {
            this.plugin.getDatabase().addKill(killer.getUniqueId().toString().replace("-", ""),
                    e.getEntity().getUniqueId().toString().replace("-", ""));
        }
        System.out.println(damageCause.getCause());
        e.setDeathMessage(null);
        switch (damageCause.getCause()) {
        case BLOCK_EXPLOSION:
            // TNT
            Bukkit.broadcast(
                    this.plugin.getMessageUtils().getDeathMessage(e.getEntity(), killer, DamageCause.BLOCK_EXPLOSION));
            break;
        case CONTACT:
            // Cactus
            Bukkit.broadcast(this.plugin.getMessageUtils().getDeathMessage(e.getEntity(), killer, DamageCause.CONTACT));
            break;
        case DROWNING:
            Bukkit.broadcast(
                    this.plugin.getMessageUtils().getDeathMessage(e.getEntity(), killer, DamageCause.DROWNING));
            break;
        case ENTITY_EXPLOSION:
            // Creeper
            Bukkit.broadcast(
                    this.plugin.getMessageUtils().getDeathMessage(e.getEntity(), killer, DamageCause.ENTITY_EXPLOSION));
            break;
        case ENTITY_ATTACK:
        case ENTITY_SWEEP_ATTACK:
            Bukkit.broadcast(
                    this.plugin.getMessageUtils().getDeathMessage(e.getEntity(), killer, DamageCause.ENTITY_ATTACK));
            break;
        case FALL:
            Bukkit.broadcast(this.plugin.getMessageUtils().getDeathMessage(e.getEntity(), killer, DamageCause.FALL));
            break;
        case FALLING_BLOCK:
            Bukkit.broadcast(
                    this.plugin.getMessageUtils().getDeathMessage(e.getEntity(), killer, DamageCause.FALLING_BLOCK));
            break;
        case FIRE:
        case FIRE_TICK:
            Bukkit.broadcast(this.plugin.getMessageUtils().getDeathMessage(e.getEntity(), killer, DamageCause.FIRE));
            break;
        case HOT_FLOOR:
            // Magma Block
            Bukkit.broadcast(
                    this.plugin.getMessageUtils().getDeathMessage(e.getEntity(), killer, DamageCause.HOT_FLOOR));
            break;
        case LAVA:
            Bukkit.broadcast(this.plugin.getMessageUtils().getDeathMessage(e.getEntity(), killer, DamageCause.LAVA));
            break;
        case LIGHTNING:
            Bukkit.broadcast(
                    this.plugin.getMessageUtils().getDeathMessage(e.getEntity(), killer, DamageCause.LIGHTNING));
            break;
        case MAGIC:
            // Potion (should be impossible)
            Bukkit.broadcast(this.plugin.getMessageUtils().getDeathMessage(e.getEntity(), killer, DamageCause.MAGIC));
            break;
        case POISON:
            Bukkit.broadcast(this.plugin.getMessageUtils().getDeathMessage(e.getEntity(), killer, DamageCause.POISON));
            break;
        case PROJECTILE:
            Bukkit.broadcast(
                    this.plugin.getMessageUtils().getDeathMessage(e.getEntity(), killer, DamageCause.PROJECTILE));
            break;
        case STARVATION:
            Bukkit.broadcast(
                    this.plugin.getMessageUtils().getDeathMessage(e.getEntity(), killer, DamageCause.STARVATION));
            break;
        case SUFFOCATION:
            // Border damage or sand/gravel
            Bukkit.broadcast(
                    this.plugin.getMessageUtils().getDeathMessage(e.getEntity(), killer, DamageCause.SUFFOCATION));
            break;
        case THORNS:
            Bukkit.broadcast(this.plugin.getMessageUtils().getDeathMessage(e.getEntity(), killer, DamageCause.THORNS));
            break;
        case VOID:
            Bukkit.broadcast(this.plugin.getMessageUtils().getDeathMessage(e.getEntity(), killer, DamageCause.VOID));
            break;
        case WITHER:
            Bukkit.broadcast(this.plugin.getMessageUtils().getDeathMessage(e.getEntity(), killer, DamageCause.WITHER));
            break;
        case CRAMMING: // Very rare so handle as default death
        case CUSTOM: // No custom damage provider so impossible
        case DRAGON_BREATH: // End is disabled so impossible
        case DRYOUT: // No fish so impossible
        case FLY_INTO_WALL: // No Elytra so impossible
        case MELTING: // No snowman so impossible
        case SUICIDE: // No access to /kill command
        default:
            Bukkit.broadcast(this.plugin.getMessageUtils().getDeathMessage(e.getEntity(), killer, DamageCause.CUSTOM)); // CUSTOM == default
            break;
        }

        // Reset health to prevent death dialog after rejoin (if unbanned)
        e.getEntity().setHealth(20d);
        e.getEntity().setGameMode(GameMode.SPECTATOR);

        this.plugin.getBorderManager().addPlayerDeath();
        this.plugin.getBanUtils().addBan(e.getEntity().getUniqueId());
        if (killer == null) {
            this.plugin.getDiscordBot().broadcastMessage(
                    this.plugin.getLangUtils().get("Discord.PlayerDeath").replace("%id%", this.plugin.getTeamManager()
                            .getDiscordIdByUuid(e.getEntity().getUniqueId().toString().replace("-", ""))));
        } else {
            this.plugin.getDiscordBot()
                    .broadcastMessage(this.plugin.getLangUtils().get("Discord.PlayerDeathKiller")
                            .replace("%id%",
                                    this.plugin.getTeamManager().getDiscordIdByUuid(
                                            e.getEntity().getUniqueId().toString().replace("-", "")))
                            .replace("%id_killer%", this.plugin.getTeamManager()
                                    .getDiscordIdByUuid(killer.getUniqueId().toString().replace("-", ""))));
        }
    }
}