package de.cliffbreak.varo.listeners;

import com.destroystokyo.paper.event.server.PaperServerListPingEvent;

import org.bukkit.Bukkit;
import org.bukkit.World.Environment;
import org.bukkit.event.EventHandler;
import org.bukkit.event.Listener;
import org.bukkit.event.player.PlayerJoinEvent;
import org.bukkit.event.player.PlayerQuitEvent;
import org.bukkit.event.server.ServerListPingEvent;
import org.bukkit.scheduler.BukkitRunnable;

import de.cliffbreak.varo.Varo;
import de.cliffbreak.varo.utils.PacketReader;

public class PlayerJoinQuitListener implements Listener {

    private final Varo plugin;

    public PlayerJoinQuitListener(final Varo plugin) {
        this.plugin = plugin;
    }

    @EventHandler
    public void onPlayerQuit(final PlayerQuitEvent e) {
        e.setQuitMessage(null);
        if (!this.plugin.getTeamManager().hasTeam(e.getPlayer())) {
            return;
        }
        if (!this.plugin.getBanUtils().isBanned(e.getPlayer().getUniqueId())) {
            Bukkit.broadcast(this.plugin.getMessageUtils().getServiceMessage(e.getPlayer().getName(),
                    "hat den Server verlassen."));
        }
        if (!e.getPlayer().getLocation().getWorld().getEnvironment().equals(Environment.THE_END)) {
            this.plugin.getPlayerCache().setPlayerLocationCache(e.getPlayer().getUniqueId().toString().replace("-", ""),
                    e.getPlayer().getLocation());
        }
        if (!e.getPlayer().getLocation().getWorld().getEnvironment().equals(Environment.THE_END)
                && !this.plugin.getBanUtils().isBanned(e.getPlayer().getUniqueId())) {
            this.plugin.getNPCManager().createClone(e.getPlayer());
        }
        this.plugin.getTeamManager().setAFK(e.getPlayer().getName());
    }

    @EventHandler
    public void onPlayerJoin(final PlayerJoinEvent e) {
        e.setJoinMessage(null);

        if (!this.plugin.getTeamManager().setTeam(e.getPlayer())) {
            e.getPlayer().kickPlayer(this.plugin.getLangUtils().get("Error.NoTeam"));
            return;
        }

        // Inject PacketReader
        final PacketReader packetReader = new PacketReader(this.plugin, e.getPlayer());
        packetReader.inject();

        Bukkit.broadcast(
                this.plugin.getMessageUtils().getServiceMessage(e.getPlayer().getName(), "hat den Server betreten."));

        new BukkitRunnable() {
            @Override
            public void run() {
                e.getPlayer().setResourcePack("http://cdn.cliffbreak.de/varo.zip",
                        "73046237cc843964dbe32fb9c76c0144cdbf90af");
            }
        }.runTaskLater(this.plugin, 5); // wait 5 ticks

        this.plugin.getPlayerUtils().showLoadingScreen(e.getPlayer(),
                this.plugin.getGameTimeManager().getLoginTimeLeft());
    }

    @EventHandler
    public void onServerListPing(final ServerListPingEvent event) {
        if (event instanceof PaperServerListPingEvent) {
            handlePaperServerListPing((PaperServerListPingEvent) event);
        } else {
            this.plugin.getLogger().info("Error: Plugin is not running on a PaperServer");
        }
    }

    private void handlePaperServerListPing(final PaperServerListPingEvent e) {
        e.setHidePlayers(true);
        e.setMotd((this.plugin.getConfig().getBoolean("Varo.Debug") ? "§c§l[DEBUG] " : "") + "§9§l" + "VARO" + "§b§l "
                + "Tag: " + this.plugin.getGameTimeManager().currentDay() + "\n§r§7" + "powered by " + "§4"
                + "Cliffbreak.de");
    }
}