package de.cliffbreak.varo.managers;

import java.time.Instant;
import java.time.LocalDateTime;
import java.time.ZoneId;

import org.bukkit.Bukkit;
import org.bukkit.World;
import org.bukkit.World.Environment;
import org.bukkit.WorldBorder;

import de.cliffbreak.varo.Varo;

public class BorderManager {

    private final Varo plugin;

    public BorderManager(final Varo plugin) {
        this.plugin = plugin;
        init();
    }

    private void init() {
        final int size = this.plugin.getConfig().getInt("Varo.Border.Size");
        final int x = this.plugin.getConfig().getInt("Varo.Border.X");
        final int z = this.plugin.getConfig().getInt("Varo.Border.Z");
        for (final World world : Bukkit.getWorlds()) {
            final Environment env = world.getEnvironment();
            if (env.equals(Environment.THE_END))
                continue;
            final WorldBorder border = world.getWorldBorder();
            border.reset();
            border.setCenter(x, z);
            border.setSize(env.equals(Environment.NORMAL) ? size : (size / 8));
            border.setWarningDistance(25);
            border.setWarningTime(0);
            border.setDamageAmount(0.2);
            border.setDamageBuffer(0);
        }
    }

    private void setBorderSize(final int size, final int time) {
        for (final World world : Bukkit.getWorlds()) {
            final Environment env = world.getEnvironment();
            if (env.equals(Environment.THE_END))
                continue;
            final WorldBorder border = world.getWorldBorder();
            border.setSize(env.equals(Environment.NORMAL) ? size : (size / 8), (long) time);
        }
    }

    public void shrinkBorder(final int playTimeSeconds) {
        final int size = this.plugin.getConfig().getInt("Varo.Border.NextSize");
        this.setBorderSize(size, playTimeSeconds);
        this.plugin.getConfig().set("Varo.Border.Size", size);
        this.plugin.saveConfig();
    }

    public LocalDateTime broadcastBorderShrink() {
        final Instant curr = Instant.now();
        this.plugin.getConfig().set("Varo.Border.LastUpdate", curr.toEpochMilli());
        int size = this.plugin.getConfig().getInt("Varo.Border.NextSize");
        size = size - 110; // 55*2 for both directions
        this.plugin.getConfig().set("Varo.Border.NextSize", size);
        this.plugin.saveConfig();
        this.plugin.getDiscordBot().broadcastMessage(
                this.plugin.getLangUtils().get("Discord.BorderUpdate").replaceAll("%size%", "" + size / 2));
        return curr.atZone(ZoneId.systemDefault()).toLocalDateTime();
    }

    public int addPlayerDeath() {
        int size = this.plugin.getConfig().getInt("Varo.Border.NextSize");
        size = size - 40; // 20*2 for both directions
        this.plugin.getConfig().set("Varo.Border.NextSize", size);
        this.plugin.saveConfig();
        return size;
    }
}