package de.cliffbreak.varo.managers;

import java.time.Duration;
import java.time.Instant;
import java.time.LocalDateTime;
import java.time.LocalTime;
import java.time.ZoneId;
import java.time.temporal.ChronoUnit;

import org.bukkit.Bukkit;
import org.bukkit.scheduler.BukkitRunnable;

import de.cliffbreak.varo.Varo;

public class GameTimeManager {

    private final Varo plugin;
    private static final int firstPlayTimeMinutes = 50;
    private static final int firstPlayTimeLoginMinutes = 10;
    private static final int protectionPeriodMinutes = 11;
    private static final LocalTime firstPlayStart = LocalTime.of(19, 30, 00);
    // private static final LocalTime firstPlayStart = LocalTime.now().plusMinutes(1);
    private static final int firstRoundPreStartMinutes = 1;
    private static final int playTimeMinutes = 30;
    private static final int playTimeLoginMinutes = 5;
    private static final LocalTime playStart = LocalTime.of(19, 30, 00);
    // private static final LocalTime playStart = LocalTime.now().plusSeconds(10);

    public GameTimeManager(final Varo plugin) {
        this.plugin = plugin;
        this.startPlayTimeTimer();
        this.startBorderTimer();
    }

    /**
     * @return current Day the Server is running
     */
    public int currentDay() {
        final Instant instant = Instant.ofEpochMilli(this.plugin.getConfig().getLong("Varo.Start"));
        final LocalDateTime start = instant.atZone(ZoneId.systemDefault()).toLocalDateTime();
        final LocalDateTime current = LocalDateTime.now();
        return (int) Duration.between(start, current).toDays() + 1;
    }

    /**
     * @return time left the Game is in LOGIN State
     */
    public int getLoginTimeLeft() {
        final LocalTime currentTime = LocalTime.now();
        final LocalTime start = this.currentDay() == 1 ? firstPlayStart : playStart;
        return (int) currentTime.until(start, ChronoUnit.SECONDS);
    }

    /**
     * @return time until the start and the game running (negative) before start > 0 and after start < 0
     */
    // TODO: BREAKING handle times after 24:00
    public int getPlayTimeDeltaSeconds() {
        final LocalTime currentTime = LocalTime.now();
        final LocalTime start = this.currentDay() == 1 ? firstPlayStart : playStart;
        return (int) currentTime.until(start, ChronoUnit.SECONDS);
    }

    private void startPlayTimeTimer() {
        new BukkitRunnable() {
            @Override
            public void run() {
                int timeDelta = getPlayTimeDeltaSeconds();
                int loginTimeSeconds = currentDay() == 1 ? firstPlayTimeLoginMinutes * 60 : playTimeLoginMinutes * 60;
                int playTimeSeconds = currentDay() == 1 ? firstPlayTimeMinutes * 60 : playTimeMinutes * 60;
                // System.out.println(timeDelta);
                if (timeDelta <= loginTimeSeconds && timeDelta > 0) {
                    plugin.getGameManager().setLoginPhase();
                } else if (currentDay() == 1 && (timeDelta <= 0 && timeDelta > firstRoundPreStartMinutes * -60)) {
                    plugin.getGameManager().setPreStart();
                } else if (currentDay() == 1 && (timeDelta <= firstRoundPreStartMinutes * -60
                        && timeDelta > protectionPeriodMinutes * -60)) {
                    plugin.getGameManager().setProtectionPhase();
                    if ((timeDelta - protectionPeriodMinutes * -60) % 60 == 0) {
                        Bukkit.broadcastMessage(plugin.getLangUtils().get("Service.Protection").replace("%minutes%",
                                "" + (timeDelta - protectionPeriodMinutes * -60) / 60));
                    } else if ((timeDelta - protectionPeriodMinutes * -60) == 1) {
                        Bukkit.broadcastMessage(plugin.getLangUtils().get("Service.ProtectionOver"));
                    }
                } else if (currentDay() > 1 && (timeDelta <= 0 && timeDelta > playTimeSeconds * -1)) {
                    plugin.getGameManager().setIngame();
                } else if (currentDay() == 1
                        && (timeDelta <= protectionPeriodMinutes * -60 && timeDelta > playTimeSeconds * -1)) {
                    plugin.getGameManager().setIngame();
                } else if (timeDelta <= playTimeSeconds * -1) {
                    plugin.getGameManager().setGameTimeOver();
                }
            }
        }.runTaskTimer(this.plugin, 20l, 20l);
    }

    private void startBorderTimer() {
        new BukkitRunnable() {
            final Instant instant = Instant.ofEpochMilli(plugin.getConfig().getLong("Varo.Border.LastUpdate"));
            LocalDateTime lastUpdate = instant.atZone(ZoneId.systemDefault()).toLocalDateTime();

            @Override
            public void run() {
                final LocalDateTime now = LocalDateTime.now();
                if (lastUpdate.getDayOfYear() < now.getDayOfYear()) {
                    if (currentDay() <= 1) {
                        System.out.println("Day 1 or smaller");
                        return;
                    }
                    if (now.getHour() == playStart.minusMinutes(30).getHour()
                            && now.getMinute() == playStart.minusMinutes(30).getMinute()) {
                        System.out.println("Shrinking Border");
                        lastUpdate = plugin.getBorderManager().broadcastBorderShrink();
                    }
                }
                if (now.getHour() == playStart.getHour() && now.getMinute() == playStart.getMinute()) {
                    System.out.println("Shrinking Border");
                    plugin.getBorderManager().shrinkBorder(playTimeMinutes * 60);
                }
            }
        }.runTaskTimer(this.plugin, 0, 60 * 20);
    }
}