package de.cliffbreak.varo.managers;

import java.io.File;
import java.io.IOException;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.UUID;

import com.mojang.util.UUIDTypeAdapter;

import org.bukkit.Bukkit;
import org.bukkit.ChatColor;
import org.bukkit.Location;
import org.bukkit.entity.Player;
import org.bukkit.scoreboard.Scoreboard;
import org.bukkit.scoreboard.Team;
import org.json.simple.JSONArray;
import org.json.simple.JSONObject;
import org.json.simple.JSONValue;
import org.json.simple.parser.ParseException;

import de.cliffbreak.varo.utils.JSONConfig;
import de.cliffbreak.varo.utils.WebUtils;

public class TeamManager {

    private final JSONConfig config;
    private final Scoreboard scoreboard;
    private final ArrayList<Team> teams;
    private static final String TEAM_PREFIX = "~";

    public TeamManager() {
        this.config = new JSONConfig(new File("plugins/CliffbreakVaro", "teams.json"), getDefaults());
        this.config.save();

        this.scoreboard = Bukkit.getScoreboardManager().getMainScoreboard();
        this.teams = new ArrayList<Team>();

        for (final Team team : this.scoreboard.getTeams()) {
            team.unregister();
        }

        this.init();
    }

    private void init() {
        final JSONArray jsonTeams = this.config.getArray("teams");
        int teamKey = -1;
        for (final Object jsonRawTeam : jsonTeams) {
            teamKey++;
            final JSONObject jsonTeam = (JSONObject) jsonRawTeam;
            final Team team, afkTeam;
            if (jsonTeam.get("name") == null) {
                continue;
            }
            if (this.scoreboard.getTeam((String) jsonTeam.get("name")) == null) {
                team = this.scoreboard.registerNewTeam((String) jsonTeam.get("name"));
            } else {
                team = this.scoreboard.getTeam((String) jsonTeam.get("name"));
            }
            if (this.scoreboard.getTeam(TEAM_PREFIX + jsonTeam.get("name")) == null) {
                afkTeam = this.scoreboard.registerNewTeam(TEAM_PREFIX + jsonTeam.get("name"));
            } else {
                afkTeam = this.scoreboard.getTeam(TEAM_PREFIX + jsonTeam.get("name"));
            }
            this.teams.add(team);
            this.teams.add(afkTeam);
            team.setPrefix(ChatColor.valueOf((String) jsonTeam.get("color")) + "§l[" + jsonTeam.get("prefix") + "] ");
            team.setAllowFriendlyFire(false);
            afkTeam.setPrefix("§8§l[AFK] ");
            team.setColor(ChatColor.WHITE);
            afkTeam.setColor(ChatColor.valueOf((String) jsonTeam.get("color")));
            for (final String entry : team.getEntries()) {
                team.removeEntry(entry);
            }
            for (final String entry : afkTeam.getEntries()) {
                team.removeEntry(entry);
            }
            String playerName = "";
            int playerKey = -1;
            for (final Object rawPlayer : (JSONArray) jsonTeam.get("players")) {
                playerKey++;
                final JSONObject player = (JSONObject) rawPlayer;
                try {
                    final String result = WebUtils
                            .GET("https://sessionserver.mojang.com/session/minecraft/profile/" + player.get("uuid")); // TODO Async
                    if (result.isEmpty()) {
                        continue;
                    }
                    final JSONObject profile = (JSONObject) JSONValue.parseWithException(result);
                    playerName = (String) profile.get("name");
                    config.updatePlayerName(teamKey, playerKey, playerName);

                } catch (IOException | ParseException e) {
                    playerName = (String) player.get("name");
                }
                team.addEntry(playerName);
            }
        }
    }

    public JSONArray getJSONTeams() {
        return this.config.getArray("teams");
    }

    public ArrayList<Team> getTeams() {
        return this.teams;
    }

    public UUID getUuidFromPlayerName(final String playerName) {
        final JSONArray jsonTeams = config.getArray("teams");
        for (final Object jsonRawTeam : jsonTeams) {
            final JSONObject jsonTeam = (JSONObject) jsonRawTeam;
            for (final Object rawPlayer : ((JSONArray) jsonTeam.get("players"))) {
                final JSONObject jsonPlayer = (JSONObject) rawPlayer;
                if (jsonPlayer.get("name").equals(playerName)) {
                    System.out.println(jsonPlayer.get("uuid"));
                    return UUIDTypeAdapter.fromString((String) jsonPlayer.get("uuid"));
                }
            }
        }
        return null;
    }

    public Team getTeamByPlayer(final String player) {
        for (final Team team : this.teams) {
            if (team.hasEntry(player)) {
                if (team.getName().startsWith("~")) {
                    for (final Team team2 : this.teams) {
                        if (team2.getName().equals(team.getName().replace("~", "")))
                            return team2;
                    }
                }
                return team;
            }
        }
        return null;
    }

    public String getDiscordIdByUuid(final String uuid) {
        final JSONArray jsonTeams = config.getArray("teams");
        for (final Object jsonRawTeam : jsonTeams) {
            final JSONObject jsonTeam = (JSONObject) jsonRawTeam;
            for (final Object rawPlayer : ((JSONArray) jsonTeam.get("players"))) {
                final JSONObject jsonPlayer = (JSONObject) rawPlayer;
                if (jsonPlayer.get("uuid").equals(uuid)) {
                    return jsonPlayer.get("discord").toString();
                }
            }
        }
        return "";
    }

    public String getPlayerByDiscordId(final String id) {
        final JSONArray jsonTeams = config.getArray("teams");
        for (final Object jsonRawTeam : jsonTeams) {
            final JSONObject jsonTeam = (JSONObject) jsonRawTeam;
            for (final Object rawPlayer : ((JSONArray) jsonTeam.get("players"))) {
                final JSONObject jsonPlayer = (JSONObject) rawPlayer;
                if (jsonPlayer.get("discord").equals(id)) {
                    return jsonPlayer.get("uuid").toString();
                }
            }
        }
        return "";
    }

    public void setPlayerSpawnByKeys(final int teamKey, final int playerKey, final Location location) {
        this.config.updatePlayerSpawn(teamKey, playerKey, location);
    }

    public void setAFK(final String player) {
        final Team oldTeam = getTeamByPlayer(player);
        for (final Team team : this.teams) {
            if (team.getName().equals(TEAM_PREFIX + oldTeam.getName())) {
                team.addEntry(player);
            }
        }
    }

    public boolean setTeam(final Player player) {
        final JSONArray jsonTeams = config.getArray("teams");
        for (final Object jsonRawTeam : jsonTeams) {
            final JSONObject jsonTeam = (JSONObject) jsonRawTeam;
            for (final Object rawPlayer : ((JSONArray) jsonTeam.get("players"))) {
                final JSONObject jsonPlayer = (JSONObject) rawPlayer;
                if (jsonPlayer.get("uuid").equals(player.getUniqueId().toString().replace("-", ""))) {
                    for (final Team team : this.teams) {
                        if (team.getName().equals(jsonTeam.get("name").toString())) {
                            team.addEntry(player.getName());
                            return true;
                        }
                    }
                }
            }
        }
        return false;
    }

    public boolean hasTeam(final Player player) {
        final JSONArray jsonTeams = config.getArray("teams");
        for (final Object jsonRawTeam : jsonTeams) {
            final JSONObject jsonTeam = (JSONObject) jsonRawTeam;
            for (final Object rawPlayer : ((JSONArray) jsonTeam.get("players"))) {
                final JSONObject jsonPlayer = (JSONObject) rawPlayer;
                if (jsonPlayer.get("uuid").equals(player.getUniqueId().toString().replace("-", ""))) {
                    return true;
                }
            }
        }
        return false;
    }

    public JSONObject getPlayerSpawn(final String uuid) {
        final JSONArray jsonTeams = config.getArray("teams");
        for (final Object jsonRawTeam : jsonTeams) {
            final JSONObject jsonTeam = (JSONObject) jsonRawTeam;
            for (final Object rawPlayer : ((JSONArray) jsonTeam.get("players"))) {
                final JSONObject jsonPlayer = (JSONObject) rawPlayer;
                if (jsonPlayer.get("uuid").equals(uuid)) {
                    return (JSONObject) jsonPlayer.get("spawn");
                }
            }
        }
        return null;
    }

    @SuppressWarnings("unchecked")
    private HashMap<String, Object> getDefaults() {
        final HashMap<String, Object> defaults = new HashMap<String, Object>();

        final JSONObject team1 = new JSONObject();
        team1.put("name", "Demo-Team");
        team1.put("color", ChatColor.RED);
        team1.put("prefix", "DMO");
        final JSONArray team1Players = new JSONArray();
        final JSONObject player = new JSONObject();
        player.put("name", "AragurLP");
        player.put("uuid", "a6019ccffe694bf780bcc3f3afa265dd");
        player.put("discord", "99820967022329856");
        team1Players.add(player);
        final JSONObject player2 = new JSONObject();
        player2.put("name", "Notch");
        player2.put("uuid", "069a79f444e94726a5befca90e38aaf5");
        player2.put("discord", "");
        team1Players.add(player2);
        team1.put("players", team1Players);

        final JSONObject team2 = new JSONObject();
        team2.put("name", "Demo-Team 2");
        team2.put("color", ChatColor.GREEN);
        team2.put("prefix", "DM2");
        final JSONArray team2Players = new JSONArray();
        final JSONObject player3 = new JSONObject();
        player3.put("name", "AragurTV");
        player3.put("uuid", "dff082fcd52d4ad2b2d8d8202aea53ec");
        player3.put("discord", "550794603754815520");
        team2Players.add(player3);
        team2.put("players", team2Players);

        final JSONArray teams = new JSONArray();
        teams.add(team1);
        teams.add(team2);
        defaults.put("teams", teams);
        return defaults;
    }
}