package de.cliffbreak.varo.utils;

import java.io.File;
import java.io.FileInputStream;
import java.io.FileWriter;
import java.io.InputStreamReader;
import java.io.PrintWriter;
import java.util.HashMap;
import java.util.TreeMap;

import com.google.gson.Gson;
import com.google.gson.GsonBuilder;

import org.bukkit.Location;
import org.json.simple.JSONArray;
import org.json.simple.JSONObject;
import org.json.simple.parser.JSONParser;

public class JSONConfig {

    private final File file;
    private JSONObject json;
    private final JSONParser parser = new JSONParser();
    private final HashMap<String, Object> defaults;

    public JSONConfig(final File file, final HashMap<String, Object> defaults) {
        this.file = file;
        this.defaults = defaults;
        reload();
    }

    public void reload() {
        try {
            if (!this.file.getParentFile().exists()) {
                this.file.getParentFile().mkdirs();
            }
            if (!this.file.exists()) {
                final PrintWriter pw = new PrintWriter(file, "UTF-8");
                pw.print("{");
                pw.print("}");
                pw.flush();
                pw.close();
            }
            this.json = (JSONObject) this.parser.parse(new InputStreamReader(new FileInputStream(this.file), "UTF-8"));
        } catch (final Exception ex) {
            ex.printStackTrace();
        }
    }

    @SuppressWarnings("unchecked")
    public boolean save() {
        try {
            final JSONObject toSave = new JSONObject();

            for (final String s : this.defaults.keySet()) {
                final Object o = this.defaults.get(s);
                if (o instanceof String) {
                    toSave.put(s, getString(s));
                } else if (o instanceof Double) {
                    toSave.put(s, getDouble(s));
                } else if (o instanceof Integer) {
                    toSave.put(s, getInteger(s));
                } else if (o instanceof JSONObject) {
                    toSave.put(s, getObject(s));
                } else if (o instanceof JSONArray) {
                    toSave.put(s, getArray(s));
                }
            }

            final TreeMap<String, Object> treeMap = new TreeMap<String, Object>(String.CASE_INSENSITIVE_ORDER);
            treeMap.putAll(toSave);

            final Gson g = new GsonBuilder().setPrettyPrinting().create();
            final String prettyJsonString = g.toJson(treeMap);

            final FileWriter fw = new FileWriter(this.file);
            fw.write(prettyJsonString);
            fw.flush();
            fw.close();

            this.reload();

            return true;
        } catch (final Exception ex) {
            ex.printStackTrace();
            return false;
        }
    }

    private String getRawData(final String key) {
        return this.json.containsKey(key) ? this.json.get(key).toString()
                : (this.defaults.containsKey(key) ? this.defaults.get(key).toString() : key);
    }

    private String getString(final String key) {
        return getRawData(key);
    }

    private double getDouble(final String key) {
        try {
            return Double.parseDouble(getRawData(key));
        } catch (final Exception ex) {
        }
        return -1;
    }

    private double getInteger(final String key) {
        try {
            return Integer.parseInt(getRawData(key));
        } catch (final Exception ex) {
        }
        return -1;
    }

    private JSONObject getObject(final String key) {
        return this.json.containsKey(key) ? (JSONObject) json.get(key)
                : (this.defaults.containsKey(key) ? (JSONObject) this.defaults.get(key) : new JSONObject());
    }

    public JSONArray getArray(final String key) {
        return this.json.containsKey(key) ? (JSONArray) json.get(key)
                : (this.defaults.containsKey(key) ? (JSONArray) this.defaults.get(key) : new JSONArray());
    }

    @SuppressWarnings("unchecked")
    public void updatePlayerName(final int teamKey, final int playerKey, final String name) {
        ((JSONObject) ((JSONArray) ((JSONObject) ((JSONArray) this.json.get("teams")).get(teamKey)).get("players"))
                .get(playerKey)).put("name", name);
        this.save();
    }

    @SuppressWarnings("unchecked")
    public void updatePlayerSpawn(final int teamKey, final int playerKey, final Location location) {
        JSONObject loc = new JSONObject();
        loc.put("world", location.getWorld().getName());
        loc.put("x", location.getX());
        loc.put("y", location.getY());
        loc.put("z", location.getZ());
        loc.put("yaw", (double) location.getYaw());
        loc.put("pitch", (double) location.getPitch());
        ((JSONObject) ((JSONArray) ((JSONObject) ((JSONArray) this.json.get("teams")).get(teamKey)).get("players"))
                .get(playerKey)).put("spawn", loc);
        this.save();
    }
}