package de.cliffbreak.varo.utils;

import java.lang.reflect.Field;
import java.util.Collection;
import java.util.List;
import java.util.Map;

import org.bukkit.Bukkit;
import org.bukkit.craftbukkit.v1_15_R1.entity.CraftPlayer;
import org.bukkit.craftbukkit.v1_15_R1.inventory.CraftItemStack;
import org.bukkit.enchantments.Enchantment;
import org.bukkit.entity.Player;
import org.bukkit.potion.PotionEffectType;

import de.cliffbreak.varo.Varo;
import de.cliffbreak.varo.enums.PlayerInteractType;
import de.cliffbreak.varo.events.PlayerInteractNPCEvent;
import io.netty.channel.Channel;
import io.netty.channel.ChannelHandlerContext;
import io.netty.handler.codec.MessageToMessageDecoder;
import net.minecraft.server.v1_15_R1.AttributeModifier;
import net.minecraft.server.v1_15_R1.EnumItemSlot;
import net.minecraft.server.v1_15_R1.GenericAttributes;
import net.minecraft.server.v1_15_R1.ItemStack;
import net.minecraft.server.v1_15_R1.Packet;

public class PacketReader {

    private final Varo plugin;
    private final Player player;
    private Channel channel;

    public PacketReader(final Varo plugin, final Player player) {
        this.plugin = plugin;
        this.player = player;
    }

    public void inject() {
        final CraftPlayer cPlayer = (CraftPlayer) this.player;
        channel = cPlayer.getHandle().playerConnection.networkManager.channel;
        channel.pipeline().addAfter("decoder", "PacketInjector", new MessageToMessageDecoder<Packet<?>>() {
            @Override
            protected void decode(final ChannelHandlerContext arg0, final Packet<?> packet, final List<Object> arg2)
                    throws Exception {
                arg2.add(packet);
                readPacket(player, packet);
            }
        });
    }

    public void uninject() {
        if (channel.pipeline().get("PacketInjector") != null) {
            channel.pipeline().remove("PacketInjector");
        }
    }

    // https://wiki.vg/Protocol#Interact_Entity
    public void readPacket(final Player player, final Packet<?> packet) {
        if (packet.getClass().getSimpleName().equalsIgnoreCase("PacketPlayInUseEntity")) {

            final int id = (int) getValue(packet, "a");
            if (!this.plugin.getNPCManager().isNPC(id)) {
                return;
            }

            try {
                final PlayerInteractType type = PlayerInteractType.valueOf(getValue(packet, "action").toString());
                final PlayerInteractNPCEvent event = new PlayerInteractNPCEvent(type, player, id,
                        calculateDamage(this.player), isCriticalHit(this.player));
                Bukkit.getServer().getPluginManager().callEvent(event);
            } catch (final Exception e) {
                // noop()
            }
        }

    }

    private double calculateDamage(final Player player) {
        double damage = 1.0; // Default 1.0 Damage (by hand)
        final ItemStack nmsItemStack = CraftItemStack.asNMSCopy(player.getInventory().getItemInMainHand());
        final Collection<AttributeModifier> attributes = nmsItemStack.getItem().a(EnumItemSlot.MAINHAND)
                .get(GenericAttributes.ATTACK_DAMAGE.getName());

        for (final AttributeModifier am : attributes) {
            damage += am.getAmount();
        }

        if (isCriticalHit(player)) {
            damage = damage * 1.5;
        }

        damage = damage * player.getAttackCooldown();

        final Map<Enchantment, Integer> enchantments = player.getInventory().getItemInMainHand().getEnchantments();

        for (final Map.Entry<Enchantment, Integer> enchantment : enchantments.entrySet()) {
            // TODO: Add more enchantments?
            if (enchantment.getKey().equals(Enchantment.DAMAGE_ALL))
                damage += 0.5 * (enchantment.getValue() + 1);
        }
        return damage;
    }

    private boolean isCriticalHit(final Player player) {
        return player.getFallDistance() > 0.0F && !player.isOnGround() && !player.getLocation().getBlock().isLiquid()
                && player.getPotionEffect(PotionEffectType.BLINDNESS) == null && player.getVehicle() == null
                && !player.isSprinting() && player.getAttackCooldown() > 0.848F;
    }

    public void setValue(final Object obj, final String name, final Object value) {
        try {
            final Field field = obj.getClass().getDeclaredField(name);
            field.setAccessible(true);
            field.set(obj, value);
        } catch (final Exception e) {
        }
    }

    public Object getValue(final Object obj, final String name) {
        try {
            final Field field = obj.getClass().getDeclaredField(name);
            field.setAccessible(true);
            return field.get(obj);
        } catch (final Exception e) {
        }
        return null;
    }

}