package de.cliffbreak.varo.utils;

import org.bukkit.Bukkit;
import org.bukkit.Location;
import org.bukkit.World;
import org.bukkit.World.Environment;
import org.bukkit.inventory.MainHand;
import org.json.simple.JSONObject;

import de.cliffbreak.varo.Varo;
import de.cliffbreak.varo.enums.PlayerCacheType;

public class PlayerCache {

    private final Varo plugin;

    public PlayerCache(final Varo plugin) {
        this.plugin = plugin;
    }

    public void setPlayerCache(final String uuid, final PlayerCacheType type, final Object value) {
        this.plugin.getDatabase().updatePlayerCache(uuid, type.toString().toLowerCase(), value);
    }

    public void setPlayerLocationCache(final String uuid, final Location loc) {
        this.setPlayerCache(uuid, PlayerCacheType.WORLD, loc.getWorld().getName());
        this.setPlayerCache(uuid, PlayerCacheType.X, loc.getX());
        this.setPlayerCache(uuid, PlayerCacheType.Y, loc.getY());
        this.setPlayerCache(uuid, PlayerCacheType.Z, loc.getZ());
        this.setPlayerCache(uuid, PlayerCacheType.YAW, loc.getYaw());
        this.setPlayerCache(uuid, PlayerCacheType.PITCH, loc.getPitch());
    }

    public Location getPlayerLocationCache(final String uuid) {
        if ((String) this.plugin.getDatabase().getPlayerCache(uuid,
                PlayerCacheType.WORLD.toString().toLowerCase()) == null) {
            JSONObject jsonSpawn = this.plugin.getTeamManager().getPlayerSpawn(uuid);
            if (jsonSpawn == null) {
                for (final World world : Bukkit.getWorlds()) {
                    if (world.getEnvironment().equals(Environment.NORMAL))
                        return world.getSpawnLocation();
                }
            } else {
                return new Location(Bukkit.getWorld((String) jsonSpawn.get("world")), (double) jsonSpawn.get("x"),
                        (double) jsonSpawn.get("y"), (double) jsonSpawn.get("z"),
                        ((Double) jsonSpawn.get("yaw")).floatValue(), ((Double) jsonSpawn.get("pitch")).floatValue());
            }
            return null;
        } else {
            return new Location(
                    Bukkit.getWorld((String) this.plugin.getDatabase().getPlayerCache(uuid,
                            PlayerCacheType.WORLD.toString().toLowerCase())),
                    (double) this.plugin.getDatabase().getPlayerCache(uuid, PlayerCacheType.X.toString().toLowerCase()),
                    (double) this.plugin.getDatabase().getPlayerCache(uuid, PlayerCacheType.Y.toString().toLowerCase()),
                    (double) this.plugin.getDatabase().getPlayerCache(uuid, PlayerCacheType.Z.toString().toLowerCase()),
                    ((Double) this.plugin.getDatabase().getPlayerCache(uuid,
                            PlayerCacheType.YAW.toString().toLowerCase())).floatValue(),
                    ((Double) this.plugin.getDatabase().getPlayerCache(uuid,
                            PlayerCacheType.PITCH.toString().toLowerCase())).floatValue());
        }
    }

    public String getName(final String uuid) {
        return (String) this.plugin.getDatabase().getPlayerCache(uuid, PlayerCacheType.NAME.toString().toLowerCase());
    }

    public int getSkinParts(final String uuid) {
        Object skinParts = this.plugin.getDatabase().getPlayerCache(uuid,
                PlayerCacheType.SKINPARTS.toString().toLowerCase());
        if (skinParts == null) {
            return 127;
        }
        return (int) skinParts;
    }

    public MainHand getMainHand(final String uuid) {
        String mainHand = (String) this.plugin.getDatabase().getPlayerCache(uuid,
                PlayerCacheType.MAINHAND.toString().toLowerCase());
        if (mainHand == null) {
            return MainHand.RIGHT;
        }
        return MainHand.valueOf(mainHand);
    }

    public byte getMainHandAsByte(final String uuid) {
        if (getMainHand(uuid) == MainHand.RIGHT)
            return (byte) 1;
        else
            return (byte) 0;
    }
}