package de.cliffbreak.varo.utils;

import com.destroystokyo.paper.Title;

import org.bukkit.Bukkit;
import org.bukkit.GameMode;
import org.bukkit.Location;
import org.bukkit.World;
import org.bukkit.World.Environment;
import org.bukkit.entity.Player;
import org.bukkit.potion.PotionEffect;
import org.bukkit.potion.PotionEffectType;
import org.bukkit.scheduler.BukkitRunnable;
import org.bukkit.scheduler.BukkitTask;

import de.cliffbreak.varo.Varo;
import de.cliffbreak.varo.enums.GameState;

public class PlayerUtils {
    private final Varo plugin;

    public PlayerUtils(final Varo plugin) {
        this.plugin = plugin;
    }

    public void showLoadingScreen(final Player p, int seconds) {
        if (seconds < 10) {
            seconds = 10; // default to 10 seconds for a smooth transition
        }
        World w = null;
        for (final World world : Bukkit.getWorlds()) {
            if (world.getEnvironment().equals(Environment.THE_END)) {
                w = world;
            }
        }

        p.teleport(new Location(w, 0, 0, 100));
        p.setFallDistance(0);
        p.setGameMode(GameMode.SPECTATOR);
        p.setFlying(true);
        p.setFlySpeed(0.0f);
        final Title title = new Title(this.plugin.getLangUtils().get("Service.LoadingTitle"),
                this.plugin.getLangUtils().get("Service.LoadingSubTitle"), 20, (seconds - 2) * 20, 20);
        p.sendTitle(title);

        p.removePotionEffect(PotionEffectType.BLINDNESS);
        p.addPotionEffect(new PotionEffect(PotionEffectType.BLINDNESS, seconds * 20, 1, false, false));

        p.setPlayerListHeaderFooter("§b------------------------------\n"
                + this.plugin.getLangUtils().get("Service.LoadingTitle") + "\n§b------------------------------\n§r ",
                "§7§o\npowered by Cliffbreak.de");

        final BukkitTask hidePlayersTask = new BukkitRunnable() {
            @Override
            public void run() {
                for (final Player player : Bukkit.getOnlinePlayers()) {
                    if (!p.equals(player)) {
                        player.hidePlayer(plugin, p);
                    }
                }
            }
        }.runTaskTimer(this.plugin, 0l, 1l);

        new BukkitRunnable() {
            @Override
            public void run() {
                final int seconds = plugin.getGameTimeManager().getLoginTimeLeft();
                if (seconds <= 0) {
                    this.cancel();
                    p.sendActionBar("");
                } else {
                    p.sendActionBar(String.format(plugin.getLangUtils().get("Service.LoadingCountdown"),
                            (int) Math.floor(seconds / 60), seconds % 60));
                }
            }
        }.runTaskTimer(this.plugin, 0, 20);

        new BukkitRunnable() {
            @Override
            public void run() {
                hidePlayersTask.cancel();
                plugin.getNPCManager().removeClone(p.getUniqueId());
                p.teleport(plugin.getPlayerCache().getPlayerLocationCache(p.getUniqueId().toString().replace("-", "")));
                if (plugin.getGameManager().getGameState().equals(GameState.PRESTART)) {
                    p.setGameMode(GameMode.ADVENTURE);
                } else {
                    p.setGameMode(GameMode.SURVIVAL);
                }
                p.setFlySpeed(0.1f);
                p.setFlying(false);
                new BukkitRunnable() {
                    @Override
                    public void run() {
                        p.removePotionEffect(PotionEffectType.INVISIBILITY);
                        for (final Player player : Bukkit.getOnlinePlayers()) {
                            p.showPlayer(plugin, player);
                            player.showPlayer(plugin, p);
                        }
                        plugin.getNPCManager().syncClones(p);
                    }
                }.runTaskLater(plugin, 10);
            }
        }.runTaskLater(this.plugin, seconds * 20);
    }

}